--Honk, version 1.0, by Pizzasgood (Rin Ganborro)
--This plugin allows you to honk your ship's horn with the /honk command
--or by simply emoting 'honks' in Sector chat, ala "T/me honks".
--It works by watching Sector chat for a player to emote "honks", and
--then playing an appropriate audio file based on their ship and their
--distance from you.  My primary motivation for writing this was that
--the Behemoth XC doesn't have any weapons ports, making it a terrebly
--boring ship to fly (unless some noob gets in your way and pulls the
--deer-in-headlights routine :D).  Besides, even space-trucks should
--have horns, to /dev/null with physics.  Ours aren't very real anyway.
--
--Keep in mind that only other players who have this plugin will be able
--to hear you when you honk.  Everybody else only sees the emote.
--
declare('honk', honk or {})
--math.randomseed( os.time() ); --seed the random number generate (this isn't needed right now, but maybe later)

-- SETTINGS
local max_vol = 1.0;
local min_vol = 0.1;

local distance_factor = 500; --smaller numbers attenuate faster (this is not currently used, but would be if you used curvey attenuation)
local max_distance = 5000; --distance to use for ships out of radar range, if only_in_range not set to true
local only_in_range = false; --when true, don't play audio for honks that happen outside radar range
-- END OF SETTINGS

local honk_message = "honks" --DO NOT CHANGE, or it won't work with other users.  This is how it detects that a player has honked.

--Load the sounds into the system
local audio_path = "plugins/honk/audio/"
gksound.GKLoadSound{soundname="behemoth_honk", filename=audio_path.."behemoth.ogg"};
gksound.GKLoadSound{soundname="centaur_honk", filename=audio_path.."centaur.ogg"};
gksound.GKLoadSound{soundname="atlas_honk", filename=audio_path.."atlas.ogg"};

--This simply sends the honk to sector chat, so you don't have to manually emote it
--Just bind 'honk' to the key you want to use as your ship's horn.
--TODO:  Have this optionally auto-bound to the weapon controls if you're in an XC (or maybe to weapon controls that don't have weapons attached regardless of ship?)
function honk.honk(_,data)
	SendChat("/me "..honk_message, "SECTOR");
end

--This function looks at each sector-chat emote to detect honks.
--When a honk is detected, it determines which audio to play and
--and what volume to use - if it should play at all.
chat_processor = {};
function chat_processor:CHAT_MSG_SECTOR_EMOTE(event, data)
	--data has 'msg', 'name', and 'faction'
	if (data.msg == honk_message) then
		local my_id = GetCharacterIDByName(GetPlayerName());
		local honker_id = GetCharacterIDByName(data.name);
		local ship = GetPrimaryShipNameOfPlayer(honker_id);
		local distance;
		if (my_id == honker_id) then
			distance = 0;
		else
			distance = GetPlayerDistance(honker_id);
		end
		if (not only_in_range and distance == nil) then
			distance = max_distance;
		end
		if (distance ~= nil) then
			--compute the volume based on distance
			local volume = (max_vol - min_vol) / (distance/distance_factor + 1) + min_vol; --curvey attenuation
			--local volume = ((min_vol-max_vol)/max_distance)*distance + max_vol; --linear attenuation
			
			--decide which sound to play
			--TODO:  Find a fog-horn clip and a way to determine if we're inside a cappy
			--TODO:  Add several versions of each ship's honk to add variety (probably too lazy to bother)
			if (string.find(ship, "Behemoth") ~= nil) then
				gksound.GKPlaySound("behemoth_honk", volume);
			elseif (string.find(ship, "Centaur") ~= nil) then
				gksound.GKPlaySound("centaur_honk", volume);
			else
				--I am too lazy to find a sound for every ship, so this will be for everything else...
				gksound.GKPlaySound("atlas_honk", volume);
			end
		end
	end
end
RegisterEvent(chat_processor, "CHAT_MSG_SECTOR_EMOTE");

RegisterUserCommand("honk",honk.honk);


--this is a test function so you can see what things sound like from different distances
--you use it like this: /testhonk 1000
--that would let you hear the horn of your current ship from 1000 m away
--it also prints out the volume number used to attenuate the honk
function honk.testhonk(_,data)
	--data[1] = distance
	if (true) then
		local my_id = GetCharacterIDByName(GetPlayerName());
		local honker_id = my_id;--GetCharacterIDByName(data.name);
		local ship = GetPrimaryShipNameOfPlayer(honker_id);
		local distance;
		if (my_id == honker_id) then
			distance = data[1];
		else
			distance = GetPlayerDistance(honker_id);
		end
		if (not only_in_range and distance == nil) then
			distance = max_distance;
		end
		if (distance ~= nil) then
			--local volume = (max_vol - min_vol) / (distance/distance_factor + 1) + min_vol; --curvey attenuation
			local volume = ((min_vol-max_vol)/max_distance)*distance + max_vol; --linear attenuation
			print(volume);
			if (string.find(ship, "Behemoth") ~= nil) then
				gksound.GKPlaySound("behemoth_honk", volume);
			elseif (string.find(ship, "Corvus") ~= nil) then
				gksound.GKPlaySound("centaur_honk", volume);
			else
				gksound.GKPlaySound("atlas_honk", volume);
			end
		end
	end
end
RegisterUserCommand("testhonk",honk.testhonk);
